package cn.kdan.cloud.pdf.office.api.payment.feign;

import cn.kdan.cloud.pdf.office.api.payment.dto.*;
import cn.kdan.cloud.pdf.office.api.payment.feign.hystrix.OrderHystrix;
import cn.kdan.cloud.pdf.office.api.payment.vo.CreateOrderResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderGiftLogVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrderUpdateVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.OrdersVO;
import cn.kdan.cloud.pdf.office.common.constant.CommonConstant;
import cn.kdan.cloud.pdf.office.common.dto.CreateUserOrderDTO;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.validation.constraints.NotNull;
import java.util.List;
import java.util.Map;


@Component
@FeignClient(value = CommonConstant.PAYMENT_PROVIDER_NAME + "/order", fallback = OrderHystrix.class)
public interface OrderApi {

    /**
     * 创建支付订单
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @PostMapping("/createOrder")
    ResultMap<CreateOrderResultVO> createOrder(@Validated @RequestBody CreateOrderDTO createOrderDTO);

    @PostMapping("/createUserOrder")
    ResultMap<Map<String, String>> createUserOrder(@Validated @RequestBody CreateUserOrderDTO createOrderDTO);

    @PostMapping("/createUserSubscription")
    ResultMap<Map<String, String>> createUserSubscription(@Validated @RequestBody CreateUserOrderDTO createUserOrderDTO);

    /**
     * 订单创建（applePay专用）
     *
     * @param createOrderDTO createOrderDTO
     * @return CreateOrderResultVO
     */
    @PostMapping("/appStoreCreateOrder")
    ResultMap<CreateOrderResultVO> appStoreCreateOrder(@Validated @RequestBody CreateOrderDTO createOrderDTO);


    /**
     * appStore 权益校验
     *
     * @param equityVerificationDTO equityVerificationDTO
     * @return UserInfoVO
     */
    @PostMapping("/appStoreEquityVerification")
    ResultMap<UserInfoVO> appStoreEquityVerification(@RequestBody EquityVerificationDTO equityVerificationDTO);
    /**
     * 用于登录之后自动绑定历史权益
     * @param appTransactionIds 历史苹果交易id
     * @param userId 用户id
     * @param appleBundleId 苹果包id用于调用苹果接口
     * @param isHaveSubscription 用户是否本身已有订阅
     * @return boolean 是否绑定了至少一个权益
     */
    @PostMapping("/bindAppleTransaction")
    ResultMap<Boolean> bindAppleTransaction(@RequestParam("appTransactionIds") List<String> appTransactionIds,@RequestParam("userId")  String userId,
                                            @RequestParam("appleBundleId") String appleBundleId,@RequestParam("isHaveSubscription") Boolean isHaveSubscription);
    /**
     * 谷歌权益校验
     * @param
     * @return
     */
    @PostMapping("/googleEquityVerification")
    ResultMap<UserInfoVO> googleEquityVerification(@RequestBody GooglePayDTO googlePayDTO);

    @PostMapping("/updateStatusPayment")
    ResultMap<Boolean> updateStatusPayment(@RequestParam("userId") String userId, @RequestParam("productId") String productId);

    /**
     * 手动创建虚拟订单
     *
     * @param createOrderManual createOrderManual
     * @return OrderUpdateVO
     */
    @PostMapping("/createOrderManual")
    ResultMap<OrderUpdateVO> createOrderManual(@Validated @RequestBody CreateOrderManualDTO createOrderManual);

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    @GetMapping("/getOrderById")
    ResultMap<OrdersVO> getOrderById(@Validated @NotNull(message = "orderId不能为空")
                                     @RequestParam("orderId") String orderId);

    @GetMapping("/getStateByOrderId")
    ResultMap<Map<String, String>> getStateByOrderId(@Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId);

    @GetMapping("/getOrderListByStatus")
    ResultMap<List<OrdersVO>> getOrderListByStatus(@Validated @NotNull(message = "订单状态不能为空") @RequestParam("orderStatus") Integer orderStatus,
                                                   @Validated @NotNull(message = "userId不能为空") @RequestParam("userId") String userId);

    @PostMapping("/closeOrder")
    ResultMap<Boolean> closeOrder(@Validated @NotNull(message = "订单id不能为空") @RequestParam("orderId") String orderId);

    /**
     * 通过id获取订单信息
     *
     * @param orderId orderId
     * @return Orders
     */
    @GetMapping("/getOrderWithUpdate")
    ResultMap<OrdersVO> getOrderWithUpdate(@Validated @NotNull(message = "orderId不能为空")
                                     @RequestParam("orderId") String orderId);

    /**
     * 通过自定义订单id获取订单信息
     *
     * @param tradeNo tradeNo
     * @return Orders
     */
    @GetMapping("/getOrderByTradeNo")
    ResultMap<List<OrdersVO>> getOrderByTradeNo(@Validated @NotNull(message = "tradeNo不能为空")
                                          @RequestParam("tradeNo") String tradeNo);

    /**
     * 手动推单
     *
     * @param chargebackOrderDTO chargebackOrderDTO
     * @return Void
     */
    @PostMapping("/chargebackOrder")
    ResultMap<Void> chargebackOrder(@RequestBody ChargebackOrderDTO chargebackOrderDTO);


    /**
     * 取消续订
     *
     * @param cancelSubscription cancelSubscription
     * @return Void
     */
    @PostMapping("/cancelSubscription")
    ResultMap<Void> cancelSubscription(@Validated @RequestBody CancelSubscriptionDTO cancelSubscription);

    /**
     * 通过用户id获取订单信息
     *
     * @param userId 用户id
     * @return Orders
     */
    @GetMapping("/getOrderByUserId")
    ResultMap<List<OrdersVO>> getOrderByUserId(@Validated @NotNull(message = "userId不能为空")
                                                @RequestParam("userId") String userId);

    /**
     * 更新赠送订阅表，并且记录赠送日志
     * @param userId 赠送人id
     * @param email 被赠送人邮箱
     * @param name 被赠送人名字
     * @return
     */
    @PostMapping("/giveAway")
    ResultMap<OrderGiftLogVO> giveAway(@RequestParam("userId") String userId, @RequestParam("email") String email, @RequestParam("name") String name);

    @GetMapping("/getGiftNum")
    ResultMap<Long> getGiftNum(@RequestParam("userId") String userId);

    @GetMapping("/getOrderGiftLogById")
    ResultMap<OrderGiftLogVO> getOrderGiftLogById(@RequestParam("orderGiftLogId")String orderGiftLogId);

    @PostMapping("/acceptUser")
    ResultMap<Void> acceptUser(@RequestParam("orderGiftLogId") String orderGiftLogId, @RequestParam("receiveUserId") String receiveUserId);
    @PostMapping("/cancelGiveAway")
    ResultMap<Boolean> cancelGiveAway(@RequestParam("orderGiftLogId") String orderGiftLogId);

    @GetMapping("/pageOrderGiftLog")
    ResultMap<Page<OrderGiftLogVO>> pageOrderGiftLog(@RequestParam("userId")String userId,
                                                     @RequestParam(name = "email",required = false)String email,
                                                     @RequestParam(name = "name",required = false)String name,
                                                     @RequestParam(name = "page",defaultValue = "1",required = false)Integer page,
                                                     @RequestParam(name ="pageSize",defaultValue = "10",required = false)Integer pageSize);
}
