package cn.kdan.cloud.pdf.office.payment.client;

import cn.hutool.core.util.StrUtil;
import cn.kdan.cloud.pdf.office.api.payment.appstore.*;
import cn.kdan.cloud.pdf.office.api.payment.constant.AppStoreAPIConstant;
import cn.kdan.cloud.pdf.office.api.payment.dto.AppStoreOrderSucceedDTO;
import cn.kdan.cloud.pdf.office.common.constant.RabbitMqConstant;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.utils.JsonUtils;
import cn.kdan.cloud.pdf.office.payment.enums.AppStoreReceiptVerifyStatusEnum;
import cn.kdan.cloud.pdf.office.payment.interceptor.AppStoreClientHttpRequestInterceptor;
import cn.kdan.cloud.pdf.office.payment.properties.AppStoreProperties;
import cn.kdan.cloud.pdf.office.payment.utils.BearerTokenAuthenticator;
import cn.kdan.cloud.pdf.office.payment.utils.IOUtils;
import lombok.extern.slf4j.Slf4j;
import org.jetbrains.annotations.NotNull;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.http.client.SimpleClientHttpRequestFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.client.DefaultResponseErrorHandler;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.util.*;

/**
 * @author ComPDFKit-WPH 2023/6/28
 */
@Component
@Slf4j
public class AppStoreClient {

    private final RestTemplate restTemplate = new RestTemplate();

    private final AppStoreProperties properties;

    private final RabbitTemplate rabbitTemplate;


    public AppStoreClient(AppStoreProperties properties, AppStoreClientHttpRequestInterceptor appStoreClientHttpRequestInterceptor, RabbitTemplate rabbitTemplate) {
        this.properties = properties;
        this.rabbitTemplate = rabbitTemplate;
        SimpleClientHttpRequestFactory factory = new SimpleClientHttpRequestFactory();
        factory.setReadTimeout(properties.getReadTimeOut());
        factory.setConnectTimeout(properties.getConnectTimeout());
        restTemplate.setRequestFactory(factory);
//        restTemplate.setInterceptors(Collections.singletonList(appStoreClientHttpRequestInterceptor));
        restTemplate.setErrorHandler(new DefaultResponseErrorHandler() {
            @Override
            public void handleError(@NotNull ClientHttpResponse clientHttpResponse) throws IOException {
                // 只要重写此方法，不去抛出HttpClientErrorException异常即可
                // HttpStatus statusCode = clientHttpResponse.getStatusCode();
            }
        });
    }

    /**
     * verifyReceipt 验证收据
     *
     * @param appStoreOrderSucceed receipt
     * @return AppStoreReceiptInfo
     */
    public AppStoreReceiptInfo verifyReceipt(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        Map<String,Object> map = new HashMap<>();
        map.put("password",properties.getPassword());
        map.put("exclude-old-transactions", true);

        AppStoreReceiptInfo body = restTemplate.exchange(properties.getAddress().concat(AppStoreAPIConstant.VERIFY_RECEIPT),
                HttpMethod.POST,
                new HttpEntity<>(JsonUtils.getJsonString(map), getJsonHttpHeaders()),
                AppStoreReceiptInfo.class).getBody();
        assert body != null;
        if (body.getStatus() != 0){
            if (body.getStatus() == 21007){
                log.info("当前订单属于sandbox数据: {}",appStoreOrderSucceed.getOrderId());
                return sandboxVerifyReceipt(appStoreOrderSucceed);
            }
            // 判断是否需要重试
            if (AppStoreReceiptVerifyStatusEnum.whetherToRetry(body.getStatus()) && appStoreOrderSucceed.isRetry()){
                // 发送重试消息到队列中 延时时间 1小时
                rabbitTemplate.convertAndSend(RabbitMqConstant.PAYMENT_DELAY_EXCHANGE,
                        RabbitMqConstant.APPSTORE_VERIFY_RECEIPT_RETRY_ROUTING_KEY,
                        JsonUtils.getJsonString(appStoreOrderSucceed),
                        (msg -> {
                            msg.getMessageProperties().setHeader("x-delay", (60 * 60 * 1000) + "");
                            return msg;
                        }));
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_NEED_TO_RETRY);

            }
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_FAILED);
        }
        return body;
    }

    public AppStoreReceiptInfo sandboxVerifyReceipt(AppStoreOrderSucceedDTO appStoreOrderSucceed) {
        Map<String,Object> map = new HashMap<>();
        map.put("password",properties.getPassword());
        map.put("exclude-old-transactions", true);

        AppStoreReceiptInfo body = restTemplate.exchange(properties.getSandboxAddress().concat(AppStoreAPIConstant.VERIFY_RECEIPT),
                HttpMethod.POST,
                new HttpEntity<>(JsonUtils.getJsonString(map), getJsonHttpHeaders()),
                AppStoreReceiptInfo.class).getBody();
        assert body != null;
        if (body.getStatus() != 0){
            // 判断是否需要重试
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_APP_STORE_TICKET_VALIDATION_FAILED);
        }
        return body;
    }

    /**
     * 创建 APPLICATION_JSON HttpHeaders
     *
     * @return HttpHeaders
     */
    @NotNull
    private HttpHeaders getJsonHttpHeaders() {
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        return httpHeaders;
    }

    private String generateBearerToken(String appBundleId) {
        String issuerId = properties.getIssuerId(); // 从properties中获取issuerId
        String keyId = properties.getKeyId(); // 从properties中获取keyId
        String bundleId = appBundleId; // 从properties中获取bundleId
        String content = new IOUtils().getAppStoreKey(); // 从IOUtils获取内容

        BearerTokenAuthenticator authenticator = new BearerTokenAuthenticator(content, keyId, issuerId, bundleId);
        return authenticator.generateToken();
    }

    // Method to fetch transaction history and return the first AppTransaction
    public AppTransaction fetchFirstAppTransaction(String transactionId, String appBundleId) {
        String bearerToken = generateBearerToken(appBundleId); // 直接调用生成Bearer token的方法

        // Create headers and add the Bearer token
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        httpHeaders.setBearerAuth(bearerToken);
        String isSandbox = properties.getIsSandbox();
        // Define the sorting order
        String sortingOrder = "?sort=DESCENDING"; // 设置为 DESCENDING 以按最新的顺序排序
        String url = "https://api.storekit-sandbox.itunes.apple.com/inApps/v2/history/{transactionId}";
        // Make the GET request with sorting order
        if (isSandbox.equals("true")){
            url = "https://api.storekit-sandbox.itunes.apple.com/inApps/v2/history/{transactionId}";
        }else{
            url = "https://api.storekit.itunes.apple.com/inApps/v2/history/{transactionId}";
        }
        // Make the GET request
        HistoryResponse response = restTemplate.exchange(
                url + sortingOrder,
                HttpMethod.GET,
                new HttpEntity<>(null, httpHeaders),
                HistoryResponse.class,
                transactionId
        ).getBody();

        if (response != null && response.getSignedTransactions() != null && !response.getSignedTransactions().isEmpty()) {
            List<AppTransaction> signedTransactions = new ArrayList<>();
            response.getSignedTransactions().forEach(item -> {
                String payloadCode = StrUtil.split(item, '.').get(1);
                String decodedString = new String(Base64.getDecoder().decode(payloadCode));
                AppTransaction appTransaction = JsonUtils.jsonStringToBean(decodedString, AppTransaction.class);
                signedTransactions.add(appTransaction);
            });
            // Return the first AppTransaction
            return signedTransactions.isEmpty() ? null : signedTransactions.get(0);
        }
        return null; // No transactions found
    }

    public AppTransaction fetchSandboxFirstAppTransaction(String transactionId, String appBundleId) {
        String bearerToken = generateBearerToken(appBundleId); // 直接调用生成Bearer token的方法

        // Create headers and add the Bearer token
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        httpHeaders.setBearerAuth(bearerToken);
        String isSandbox = properties.getIsSandbox();
        // Define the sorting order
        String sortingOrder = "?sort=DESCENDING"; // 设置为 DESCENDING 以按最新的顺序排序
        String url = "https://api.storekit-sandbox.itunes.apple.com/inApps/v2/history/{transactionId}";
        // Make the GET request
        HistoryResponse response = restTemplate.exchange(
                url + sortingOrder,
                HttpMethod.GET,
                new HttpEntity<>(null, httpHeaders),
                HistoryResponse.class,
                transactionId
        ).getBody();

        if (response != null && response.getSignedTransactions() != null && !response.getSignedTransactions().isEmpty()) {
            List<AppTransaction> signedTransactions = new ArrayList<>();
            response.getSignedTransactions().forEach(item -> {
                String payloadCode = StrUtil.split(item, '.').get(1);
                String decodedString = new String(Base64.getDecoder().decode(payloadCode));
                AppTransaction appTransaction = JsonUtils.jsonStringToBean(decodedString, AppTransaction.class);
                signedTransactions.add(appTransaction);
            });
            // Return the first AppTransaction
            return signedTransactions.isEmpty() ? null : signedTransactions.get(0);
        }
        return null; // No transactions found
    }


    public static void main(String[] args) {
        testID();
    }

    private static void testID() {
        String issuerId = "69a6de93-a322-47e3-e053-5b8c7c11a4d1";
        String keyId = "7ZM85LHZYB";
        String bundleId = "com.brother.pdfreaderprofree.mac";
        String content = new IOUtils().getAppStoreKey();
        BearerTokenAuthenticator authenticator = new BearerTokenAuthenticator(content, keyId, issuerId, bundleId);
        String bearerToken = authenticator.generateToken();
        System.out.println(bearerToken);

        // Create headers and add the Bearer token
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.setContentType(MediaType.APPLICATION_JSON);
        httpHeaders.setBearerAuth(bearerToken); // Set the Bearer token directly


        RestTemplate restTemplate = new RestTemplate();
        restTemplate.setErrorHandler(new DefaultResponseErrorHandler() {
            @Override
            public void handleError(@NotNull ClientHttpResponse clientHttpResponse) throws IOException {
                // You can handle error responses here
            }
        });
        // Define the sorting order
        String sortingOrder = "DESCENDING"; // 设置为 DESCENDING 以按最新的顺序排序

        // Make the GET request with sorting order
        HistoryResponse a = restTemplate.exchange(
                "https://api.storekit-sandbox.itunes.apple.com/inApps/v2/history/{transactionId}?sort=" + sortingOrder,
                HttpMethod.GET, // Change to GET
                new HttpEntity<>(null, httpHeaders), // No body for GET, just headers
                HistoryResponse.class,
                "2000000828948737" // Make sure to replace with the actual transactionId
        ).getBody();
        List<AppTransaction> signedTransactions = new ArrayList<>();
        a.getSignedTransactions().stream().forEach(item->{
            String payloadCode = StrUtil.split(item, '.').get(1);
            String decodedString = new String(Base64.getDecoder().decode(payloadCode));
            System.out.println(decodedString);
            AppTransaction appTransaction = JsonUtils.jsonStringToBean(decodedString, AppTransaction.class);
            signedTransactions.add(appTransaction);
        });
        signedTransactions.get(1).getExpiresDate();
        System.out.println(a);
    }

}
