package cn.kdan.cloud.pdf.office.payment.controller;

import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionDTO;
import cn.kdan.cloud.pdf.office.api.payment.dto.CreateSubscriptionManualDTO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeResultVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionSchemeVO;
import cn.kdan.cloud.pdf.office.api.payment.vo.SubscriptionsVO;
import cn.kdan.cloud.pdf.office.api.product.enums.PlatformEnum;
import cn.kdan.cloud.pdf.office.common.pojo.ResultMap;
import cn.kdan.cloud.pdf.office.payment.service.SubscriptionsService;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * @author ComPDFKit-WPH 2023/2/6
 * <p>
 * SubscriptionController
 */
@RestController
@RequestMapping("/subscription")
@RequiredArgsConstructor
public class SubscriptionController {

    private final SubscriptionsService subscriptionsService;

    /**
     * 创建订阅信息
     *
     * @param subscriptionInfo subscriptionInfo
     * @return 订阅id
     */
    @PostMapping("/createSubscription")
    public ResultMap<String> createSubscription(@Validated @RequestBody CreateSubscriptionDTO subscriptionInfo) {
        return ResultMap.success(subscriptionsService.createSubscription(subscriptionInfo));
    }

    /**
     * 通过用户id和产品id 获取最新的订阅信息
     *
     * @param userId userId
     * @param platform  platform
     * @return Subscriptions
     */
    @GetMapping("/getSubscriptions")
    public ResultMap<SubscriptionsVO> getSubscriptions(@Validated @NotNull(message = "userId 不能为空") @RequestParam("userId") String userId,
                                                       @Validated @NotNull(message = "platform 不能为空") @RequestParam("platform") Integer platform) {
        return ResultMap.success(subscriptionsService.getSubscriptions(userId, PlatformEnum.getEnumByValue(platform)));
    }

    /**
     * 通过用户id和产品id 获取最新的订阅信息
     *
     * @param userId userId
     * @param productId  platform
     * @return Subscriptions
     */
    @GetMapping("/getSubscriptionsByProductId")
    public ResultMap<SubscriptionsVO> getSubscriptionsByProductId(@Validated @NotNull(message = "userId 不能为空") @RequestParam("userId") String userId,
                                                       @Validated @NotNull(message = "productId 不能为空") @RequestParam("productId") String productId) {
        return ResultMap.success(subscriptionsService.getSubscriptionsByProductId(userId, productId));
    }
    /**
     * 查询最新一条实际支付的订阅信息（排除赠送的订阅）
     * @param userId 用户id
     * @param productId 产品id
     * @return SubscriptionsVO
     */
    @GetMapping("/getLatestSubscriptionForActualPayment")
    public ResultMap<SubscriptionsVO> getLatestSubscriptionForActualPayment(@Validated @NotNull(message = "userId 不能为空") @RequestParam("userId") String userId,
                                                                  @Validated @NotNull(message = "productId 不能为空") @RequestParam("productId") String productId) {
        return ResultMap.success(subscriptionsService.getLatestSubscriptionForActualPayment(userId, productId));
    }
    /**
     * 查询当前用户的订阅计划和支付订单
     *
     * @param userId   userId
     * @return SubscriptionSchemeVO
     */
    @GetMapping("/getSubscriptionSchemeInfo")
    public ResultMap<Map<String,SubscriptionSchemeVO>> getSubscriptionSchemeInfo(@Validated @NotNull(message = "用户id不能为空") @RequestParam String userId,
                                                                                 @RequestParam String appId) {
        return ResultMap.success(subscriptionsService.getSubscriptionSchemeInfo(userId,appId));
    }


    /**
     * 获取订阅
     *
     * @param userId userId
     * @param productId productId
     * @return List
     */
    @GetMapping("/getSubscriptionsByUserIdAndProductId")
    ResultMap<List<SubscriptionsVO>> getSubscriptionsByUserIdAndProductId(@RequestParam("userId") String userId,
                                                                          @RequestParam("productId") String productId) {
        return ResultMap.success(subscriptionsService.getSubscriptionsByUserIdAndProductId(userId,productId));
    }

    /**
     * 获取用户下次扣款时间
     *
     * @param userId userId
     * @param productId productId
     * @return List
     */
    @GetMapping("/getNextPayTime")
    ResultMap<LocalDateTime> getNextPayTime(@RequestParam("userId") String userId,
                                            @RequestParam("productId") String productId) {
        return ResultMap.success(subscriptionsService.getNextPayTime(userId,productId));
    }

    /**
     * 判断是否可以送试用
     * @param userId 用户id
     * @return Boolean
     */
    @GetMapping("/judgeTrail")
    ResultMap<Boolean> judgeTrail(@RequestParam("userId") String userId) {
        return ResultMap.success(subscriptionsService.judgeTrail(userId));
    }
    //判断用户是否有ai折扣
    @GetMapping("/judgeAiDiscount")
    ResultMap<Boolean> judgeAiDiscount(@RequestParam("userId") String userId) {
        return ResultMap.success(subscriptionsService.judgeAiDiscount(userId));
    }
    @GetMapping("/judgeAiTrail")
    ResultMap<Boolean> judgeAiTrail(@RequestParam("userId") String userId) {
        return ResultMap.success(subscriptionsService.judgeAiTrail(userId));
    }

}
