package cn.kdan.cloud.pdf.office.website.service.impl;

import cn.hutool.extra.qrcode.QrCodeUtil;
import cn.kdan.cloud.pdf.office.api.account.feign.UserApi;
import cn.kdan.cloud.pdf.office.api.payment.enums.PaymentMethodEnum;
import cn.kdan.cloud.pdf.office.api.payment.feign.OrderApi;
import cn.kdan.cloud.pdf.office.api.product.feign.ProductApi;
import cn.kdan.cloud.pdf.office.api.product.vo.ListingProductVO;
import cn.kdan.cloud.pdf.office.api.product.vo.ProductVO;
import cn.kdan.cloud.pdf.office.common.dto.CreateUserOrderDTO;
import cn.kdan.cloud.pdf.office.common.enums.ExceptionEnum;
import cn.kdan.cloud.pdf.office.common.enums.account.PDFOfficeUserStatusEnum;
import cn.kdan.cloud.pdf.office.common.exception.BackendRuntimeException;
import cn.kdan.cloud.pdf.office.common.pojo.CustomUserDetails;
import cn.kdan.cloud.pdf.office.common.vo.UserInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserSubscriptionInfoVO;
import cn.kdan.cloud.pdf.office.common.vo.UserVO;
import cn.kdan.cloud.pdf.office.website.service.OrderV2Service;
import cn.kdan.cloud.pdf.office.website.service.ProductInfoService;
import cn.kdan.cloud.pdf.office.website.utils.SecurityUtils;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author ComPDFKit-WPH 2023/2/9
 */
@Service
@Slf4j
@RequiredArgsConstructor
@Transactional
public class OrderV2ServiceImpl implements OrderV2Service {

    private final OrderApi orderApi;

    private final UserApi userApi;

    private final ProductApi productApi;

    private final ProductInfoService productInfoService;

    @Override
    public Map<String, String> createOrder(CreateUserOrderDTO createOrderDTO) {
        createOrderDTO.setDiscountFlag(0);
        CustomUserDetails user = SecurityUtils.getUser();
        createOrderDTO.setUserId(user.getId());
        // 判断用户状态
        UserVO userVO = userApi.getById(user.getId()).getResult();
        if (userVO.getValidFlag().equals(PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString())) {
            log.error("用户注销中，无法购买，{}", user.getId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ABNORMAL_USER_STATUS);
        }
        // 获取商品信息
        ProductVO productVO = productApi.getProduct(createOrderDTO.getProductId()).getResult();
        // 优惠券校验
        boolean couponFlag = false;
        // 批量购买时，校验价格
        if (createOrderDTO.getNum() > 1) {
            // 查询商品及批量价格
            Map<String, String> batchProductPrice = productInfoService.getBatchProductPrice(createOrderDTO.getProductId(), createOrderDTO.getNum());
            // 判断币种
            BigDecimal totalPrice;
            if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PAYPAL.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PADDLE.getValue())) {
                totalPrice = new BigDecimal(batchProductPrice.get("totalPrice"));
            } else {
                totalPrice = new BigDecimal(batchProductPrice.get("cnyTotalPrice"));
            }
            if (totalPrice.compareTo(createOrderDTO.getPrice()) != 0) {
                log.error("批量购买价格校验失败，商品id：{}，购买数量：{}，批量计算价格：{}，用户传入价格：{}", createOrderDTO.getProductId(), createOrderDTO.getNum(), batchProductPrice, createOrderDTO.getPrice());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
            }
            createOrderDTO.setDiscountFlag(4);
        } else {
            // 单个购买，根据商品类型，校验价格
            if (StringUtils.isNotEmpty(createOrderDTO.getCouponCode())) {
                ListingProductVO checkCoupon = productInfoService.checkCoupon(createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getCouponCode()).getResult();
                if (null == checkCoupon) {
                    log.error("优惠券校验失败，code:{}", createOrderDTO.getCouponCode());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
                }
                // 根据支付方式验证价格
                if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PAYPAL.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PADDLE.getValue())) {
                    if (checkCoupon.getDisplayPrice() == null || checkCoupon.getDisplayPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                        log.error("购买时使用优惠券校验美元价格失败，商品id：{}，用户id：{}，用户传入价格：{}, 优惠券信息价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getPrice(), checkCoupon);
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
                    }
                } else {
                    if (checkCoupon.getCnyDisplayPrice() == null || checkCoupon.getCnyDisplayPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                        log.error("购买时使用优惠券校验人民币价格失败，商品id：{}，用户id：{}，用户传入价格：{}, 优惠券信息价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getPrice(), checkCoupon);
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
                    }
                }
                createOrderDTO.setDiscountFlag(1);
                // 验证优惠券成功，标记更新优惠券状态
                couponFlag = true;
            } else {
                // 教育优惠和原价，判断商品价格，活动商品判断优惠价
                BigDecimal price = !ObjectUtils.isEmpty(productVO.getDisplayPrice()) ? productVO.getDisplayPrice() : productVO.getPrice();
                BigDecimal cnyPrice = !ObjectUtils.isEmpty(productVO.getCnyDisplayPrice()) ? productVO.getCnyDisplayPrice() : productVO.getCnyPrice();
                // 根据支付方式验证价格
                if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PAYPAL.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.PADDLE.getValue())) {
                    // paypal paddle 美元
                    if (price == null || price.compareTo(createOrderDTO.getPrice()) != 0) {
                        log.error("购买美元价格校验失败，商品id：{}，用户id：{}，商品价格：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), price, createOrderDTO.getPrice());
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                    }
                } else {
                    // 微信 支付宝 人民币
                    if (cnyPrice == null || cnyPrice.compareTo(createOrderDTO.getPrice()) != 0) {
                        log.error("购买人民币价格校验失败，商品id：{}，用户id：{}，商品价格：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), cnyPrice, createOrderDTO.getPrice());
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                    }
                }
            }
        }
        // 价格校验通过，调用order创建
        Map<String, String> result = orderApi.createUserOrder(createOrderDTO).getResult();
        String payHref = result.get("payHref");
        if (StringUtils.isEmpty(payHref)) {
            log.error("创建订单失败，{}", createOrderDTO);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_CREATE_FAILED);
        }
        if (couponFlag) {
            // 更新优惠券状态
            productInfoService.updateCouponState(createOrderDTO.getCouponCode(), 2, result.get("orderId"));
        }
        log.info("创建订单成功，{}", result);
        // 微信支付宝链接生成二维码
        if (createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.ALIPAY.getValue()) || createOrderDTO.getPaymentMethod().equals(PaymentMethodEnum.WXPAY.getValue())) {
            String qrcode = generateQrcode(payHref);
            result.put("qrcode", qrcode);
        }
        return result;
    }

    private static String generateQrcode(String payHref) {
        log.info("生成二维码，保存为临时文件：{}", payHref);
        // 生成二维码并保存为临时文件
        String qrcodeFileUrl = System.getProperty("java.io.tmpdir") + "/qrcode-" + UUID.randomUUID().toString().replace("-", "") + ".png";
        log.info("二维码文件路径：{}", qrcodeFileUrl);
        File qrCodeFile = new File(qrcodeFileUrl);
        QrCodeUtil.generate(payHref, 300, 300, qrCodeFile);
        try {
            log.info("将二维码图片转换为 Base64 编码");
            // 读取二维码图片
            BufferedImage bufferedImage = ImageIO.read(qrCodeFile);
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            // 将图片写入输出流
            ImageIO.write(bufferedImage, "png", baos);
            baos.flush();
            byte[] bytes = baos.toByteArray();
            String base64 = Base64.getEncoder().encodeToString(bytes);
            return "data:image/png;base64," + base64;
        } catch (Exception e) {
            log.error("生成二维码失败，{}", e.getMessage());
            return null;
        } finally {
            // 删除临时文件
            log.info("删除二维码临时文件");
            if (qrCodeFile.exists()) {
                if (qrCodeFile.delete()) {
                    log.info("删除二维码临时文件成功");
                }
            }
        }
    }

    @Override
    public Map<String, String> createSubscription(CreateUserOrderDTO createOrderDTO) {
        createOrderDTO.setNum(1);
        createOrderDTO.setDiscountFlag(0);
        // 订阅目前支持 PayPal paddle
        if (createOrderDTO.getPaymentMethod() != 0 && createOrderDTO.getPaymentMethod() != 3) {
            log.error("订阅目前只支持 paypal 和 paddle，{}", createOrderDTO);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_CREATE_ORDER_PARAMETER);
        }
        CustomUserDetails user = SecurityUtils.getUser();
        createOrderDTO.setUserId(user.getId());
        // 判断用户状态
        UserInfoVO userVO = userApi.getMemberInfoById(user.getId()).getResult();
        if (userVO.getValidFlag().equals(PDFOfficeUserStatusEnum.LOGGED_OUT_ING.value().toString())) {
            log.error("用户注销中，无法购买，{}", user.getId());
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ABNORMAL_USER_STATUS);
        }
        // 获取商品信息
        ProductVO productVO = productApi.getProduct(createOrderDTO.getProductId()).getResult();
        UserSubscriptionInfoVO vipSubscriptionInfoVO = null;
        // 如果商品是升级，判断用户是否有升级权限
        if (productVO.getDiscountType() == 4) {
            // 筛选订单购买的产品中有没有vip订阅类型
            boolean hasVip = false;
            boolean hasSVip = false;
            // 买断版支持平台数
            int permanentPlatform = 0;
            List<UserSubscriptionInfoVO> userSubscriptionInfoVOList = userVO.getSubscriptionInfoList().stream().filter(Objects::nonNull).filter(s -> s.getPlatform() != 0 && s.getPlatform() != 3 && s.getStatus() != 2 && s.getStatus() != 4).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(userSubscriptionInfoVOList)) {
                for (UserSubscriptionInfoVO userSubscriptionInfoVO : userSubscriptionInfoVOList) {
                    // 买断的产品，判断是否多平台
                    if (userSubscriptionInfoVO.getPaymentModel().equals("2") && StringUtils.isNotEmpty(userSubscriptionInfoVO.getPlatforms())) {
                        if (userSubscriptionInfoVO.getPlatforms().contains("Mac")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("Windows")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("iOS")) permanentPlatform++;
                        if (userSubscriptionInfoVO.getPlatforms().contains("Android")) permanentPlatform++;
                    } else {
                        // 优先判断高级版
                        if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("3")) {
                            hasSVip = true;
                        } else if (StringUtils.isNotEmpty(userSubscriptionInfoVO.getLevels())&&userSubscriptionInfoVO.getLevels().contains("2")) {
                            hasVip = true;
                            vipSubscriptionInfoVO = userSubscriptionInfoVO;
                        }
                    }
                }
            }
            // 判断商品升级类型
            switch (productVO.getCode()) {
                // 标准订阅升级
                case "advanced-annual-subscription-standard-year-upgrade":
                    if (!hasVip) {
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                    }
                    createOrderDTO.setDiscountFlag(6);
                    break;
                // 单平台买断升级
                case "advanced-annual-subscription-single-upgrade":
                    if (permanentPlatform == 0) {
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                    }
                    createOrderDTO.setDiscountFlag(2);
                    break;
                // 双平台买断升级
                case "advanced-annual-subscription-multi-upgrade":
                    if (permanentPlatform < 2) {
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                    }
                    createOrderDTO.setDiscountFlag(2);
                    break;
                // 买断版的教育升级
                case "advanced-annual-subscription-edu-upgrade":
                    if (permanentPlatform == 0) {
                        throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                    }
                    createOrderDTO.setDiscountFlag(2);
                    break;
                default:
                    log.error("商品升级code错误不支持，{}", productVO);
            }
        }
        // 优惠券
        boolean couponFlag = false;
        if (StringUtils.isNotEmpty(createOrderDTO.getCouponCode())) {
            ListingProductVO checkCoupon = productInfoService.checkCoupon(createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getCouponCode()).getResult();
            if (null == checkCoupon) {
                log.error("优惠券校验失败，code:{}", createOrderDTO.getCouponCode());
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_NOT_EXISTS);
            }
            // 订阅只校验美元价格
            if (checkCoupon.getDisplayPrice() == null || checkCoupon.getDisplayPrice().compareTo(createOrderDTO.getPrice()) != 0) {
                log.error("订阅使用优惠券校验美元价格失败，商品id：{}，用户id：{}，用户购买价格：{}, 优惠券信息价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), createOrderDTO.getPrice(), checkCoupon);
                throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_COUPON_PRODUCT_ERROR);
            }
            createOrderDTO.setDiscountFlag(1);
            // 验证优惠券成功，标记更新优惠券状态
            couponFlag = true;
        } else {
            if (productVO.getCode().equals("advanced-annual-subscription-standard-year-upgrade")) {
                if (com.baomidou.mybatisplus.core.toolkit.ObjectUtils.isEmpty(vipSubscriptionInfoVO)) {
                    log.error("标准订阅升级，用户标准权益错误");
                }
                // 计算用户订阅的endDate到现在的天数
                long diff = ChronoUnit.DAYS.between(LocalDate.now(), vipSubscriptionInfoVO.getEndDate().toInstant().atZone(ZoneId.systemDefault()).toLocalDate());
                log.info("用户{}标准订阅升级，用户标准权益剩余天数为 {} 天", userVO.getEmail(), diff);
                if (diff <= 0) {
                    log.error("标准版订阅升级剩余天数错误");
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                }
                // 计算美元差价, (99.99 - 79.99 ) / 365 * 天数
                BigDecimal divide = productVO.getPrice().subtract(new BigDecimal("79.99")).multiply(new BigDecimal(diff))
                        .divide(new BigDecimal("365"), 2, RoundingMode.HALF_UP);
                if (divide == null || divide.compareTo(createOrderDTO.getPrice()) != 0) {
                    log.error("标准订阅升级价格校验失败，商品id：{}，用户id：{}，差价：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), divide, createOrderDTO.getPrice());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                }
                DateTimeFormatter formatter = DateTimeFormatter.ISO_INSTANT;
                createOrderDTO.setSubscriptionDate(vipSubscriptionInfoVO.getEndDate().toInstant().atZone(ZoneId.systemDefault()).format(formatter));
            } else {
                // 校验价格，教育优惠和原价，判断商品价格，活动和试用商品判断优惠价
                BigDecimal price = !ObjectUtils.isEmpty(productVO.getDisplayPrice()) ? productVO.getDisplayPrice() : productVO.getPrice();
                if (price == null || price.compareTo(createOrderDTO.getPrice()) != 0) {
                    log.error("订阅价格校验失败，商品id：{}，用户id：{}，商品价格：{}，用户购买价格：{}", createOrderDTO.getProductId(), createOrderDTO.getUserId(), price, createOrderDTO.getPrice());
                    throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_PRICE_ERROR);
                }
            }
        }
        // 价格校验通过，调用order创建
        Map<String, String> result = orderApi.createUserSubscription(createOrderDTO).getResult();
        if (StringUtils.isEmpty(result.get("payHref"))) {
            log.error("创建订阅订单失败，{}", createOrderDTO);
            throw new BackendRuntimeException(ExceptionEnum.EXCEPTION_MSG_ORDER_CREATE_FAILED);
        }
        if (couponFlag) {
            // 更新优惠券状态
            productInfoService.updateCouponState(createOrderDTO.getCouponCode(), 2, result.get("orderId"));
        }
        return result;
    }

}
